/*******************************************************************************
 * HY16F198
 * -----------------------------------------------------------------------------
 * Copyright 2014 HYCON Technology
 * http://www.hycontek.com/
 *
 * Library release V0.1
 *     2014/11/07
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F198.h"
#include "DrvADC.h"
#include "DrvClock.h"
#include "DrvGPIO.h"
#include "DrvLCD.h"
#include "DrvPMU.h"
#include "ModuleID.h"
#include "Sysinfra.h"
#include "System.h"
#include "Display.h"
#include "my define.h"
#include "stdlib.h"
#include "hyprintf.h"
#include "ctype.h"
#include "DrvUART.h"
#include "ACFunction.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;
int Count;

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void InitalADC(void);
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/
int main(void)
{
  long long AC_Value;

  DisplayInit();
  ClearLCDframe();
  Delay(10000);
  DisplayHYcon();
  Delay(1000);
  MCUSTATUSbits._byte = 0;
  Count=0;
  InitalADC();
  SYS_EnableGIE(7,0x1FF);  //Enable GIE(Global Interrupt)

  while(1)
  {
    if(MCUSTATUSbits.b_ADCdone)  //b_ADCdone=1 execute below
    {
      MCUSTATUSbits.b_ADCdone=0;
      AC_Value = AC_Algorithm();  // To do AC algorithm and to show current value
      AC_Value=AC_Value/0.5770;  // Using 60HZ gain value, calibrate at 2000mA
      LCD_DATA_DISPLAY(AC_Value);   //Display AC Value
      Count=0;
      DrvADC_CombFilter(0);
      DrvADC_ClearIntFlag();
      DrvADC_EnableInt();
      DrvADC_CombFilter(1);
    }
  }
  return 0;
}

/*--------------------------------------------------------------------*/
/* ADC Interrupt Subroutines                                          */
/*--------------------------------------------------------------------*/
void HW2_ISR(void)
{
  int ADCData;

  if(DrvADC_ReadIntFlag())
  {
    DrvADC_ClearIntFlag();
    ADCData=DrvADC_GetConversionData();
    AC_DataCount(Count++,ADCData);  // AC Algorithm : to get ADCData
    if(Count>=AC_DataLen)  //to do 4096 times
    {
      DrvADC_DisableInt();
      MCUSTATUSbits.b_ADCdone=1;
    }
  }
}

/*--------------------------------------------------------------------*/
/* ADC Initialization Subroutines                                     */
/*--------------------------------------------------------------------*/
void InitalADC(void)
{
  //Set ADC input pin
  DrvADC_SetADCInputChannel(ADC_Input_AIO1,ADC_Input_AIO0); //Set the ADC positive/negative input voltage source.
  DrvADC_InputSwitch(OPEN);                //ADC signal input (positive and negative) short(VISHR) control.
  DrvADC_RefInputShort(OPEN);              //Set the ADC reference input (positive and negative) short(VRSHR) control.

  DrvADC_Gain(ADC_PGA_Disable,ADC_PGA_Disable); //Input signal gain for modulator.
  DrvADC_DCoffset(0);                      //DC offset input voltage selection (VREF=REFP-REFN)
  DrvADC_RefVoltage(VDDA,VSSA);  //Set the ADC reference voltage.
  DrvADC_FullRefRange(1);                  //Set the ADC full reference range select.
                                           //0: Full reference range input
                                           //1: 1/2 reference range input
  DrvADC_OSR(10);      //10 : OSR=32
  DrvADC_CombFilter(ENABLE);   //Enable OSR
  DrvADC_ClkEnable(0,1);       //Setting ADC CLOCK ADCK=HS_CK/6 & Rising edge is high

  //Set VDDA voltage
  DrvPMU_VDDA_LDO_Ctrl(E_VDD3V);
  DrvPMU_BandgapEnable();
  DrvPMU_REFO_Enable();
  DrvPMU_AnalogGround(ENABLE);  //ADC analog ground source selection.
                                //1 : Enable buffer and use internal source(need to work with ADC)
  //Set ADC interrupt
  DrvADC_EnableInt();
  DrvADC_ClearIntFlag();
  DrvADC_Enable();
}

/*---------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                */
/*---------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
